/*
 * The main tables describing the instructions is essentially a copy
 * of the "Opcode Map" chapter (Appendix A) of the Intel 80386
 * Programmers Manual.  Usually, there is a capital letter, followed
 * by a small letter.  The capital letter tell the addressing mode,
 * and the small letter tells about the operand size.  Refer to 
 * the Intel manual for details.
 */

#define Eb OP_E, b_mode
#define indirEb OP_indirE, b_mode
#define Gb OP_G, b_mode
#define Ev OP_E, v_mode
#define indirEv OP_indirE, v_mode
#define Ew OP_E, w_mode
#define Ma OP_E, v_mode
#define M OP_E, 0
#define Mp OP_E, 0		/* ? */
#define Gv OP_G, v_mode
#define Gw OP_G, w_mode
#define Rw OP_rm, w_mode
#define Rd OP_rm, d_mode
#define Ib OP_I, b_mode
#define sIb OP_sI, b_mode	/* sign extened byte */
#define Iv OP_I, v_mode
#define Iw OP_I, w_mode
#define Jb OP_J, b_mode
#define Jv OP_J, v_mode
#define ONE OP_ONE, 0
#define Cd OP_C, d_mode
#define Dd OP_D, d_mode
#define Td OP_T, d_mode

#define eAX OP_REG, eAX_reg
#define eBX OP_REG, eBX_reg
#define eCX OP_REG, eCX_reg
#define eDX OP_REG, eDX_reg
#define eSP OP_REG, eSP_reg
#define eBP OP_REG, eBP_reg
#define eSI OP_REG, eSI_reg
#define eDI OP_REG, eDI_reg
#define AL OP_REG, al_reg
#define CL OP_REG, cl_reg
#define DL OP_REG, dl_reg
#define BL OP_REG, bl_reg
#define AH OP_REG, ah_reg
#define CH OP_REG, ch_reg
#define DH OP_REG, dh_reg
#define BH OP_REG, bh_reg
#define AX OP_REG, ax_reg
#define DX OP_REG, dx_reg
#define indirDX OP_REG, indir_dx_reg

#define Sw OP_SEG, w_mode
#define Ap OP_DIR, lptr
#define Av OP_DIR, v_mode
#define Ob OP_OFF, b_mode
#define Ov OP_OFF, v_mode
#define Xb OP_DSSI, b_mode
#define Xv OP_DSSI, v_mode
#define Yb OP_ESDI, b_mode
#define Yv OP_ESDI, v_mode

#define es OP_REG, es_reg
#define ss OP_REG, ss_reg
#define cs OP_REG, cs_reg
#define ds OP_REG, ds_reg
#define fs OP_REG, fs_reg
#define gs OP_REG, gs_reg

int OP_E(), OP_indirE(), OP_G(), OP_I(), OP_sI(), OP_REG();
int OP_J(), OP_SEG();
int OP_DIR(), OP_OFF(), OP_DSSI(), OP_ESDI(), OP_ONE(), OP_C();
int OP_D(), OP_T(), OP_rm();


#define b_mode 1
#define v_mode 2
#define w_mode 3
#define d_mode 4

#define es_reg 100
#define cs_reg 101
#define ss_reg 102
#define ds_reg 103
#define fs_reg 104
#define gs_reg 105
#define eAX_reg 107
#define eCX_reg 108
#define eDX_reg 109
#define eBX_reg 110
#define eSP_reg 111
#define eBP_reg 112
#define eSI_reg 113
#define eDI_reg 114

#define lptr 115

#define al_reg 116
#define cl_reg 117
#define dl_reg 118
#define bl_reg 119
#define ah_reg 120
#define ch_reg 121
#define dh_reg 122
#define bh_reg 123

#define ax_reg 124
#define cx_reg 125
#define dx_reg 126
#define bx_reg 127
#define sp_reg 128
#define bp_reg 129
#define si_reg 130
#define di_reg 131

#define indir_dx_reg 150

#define GRP1b NULL, NULL, 0
#define GRP1S NULL, NULL, 1
#define GRP1Ss NULL, NULL, 2
#define GRP2b NULL, NULL, 3
#define GRP2S NULL, NULL, 4
#define GRP2b_one NULL, NULL, 5
#define GRP2S_one NULL, NULL, 6
#define GRP2b_cl NULL, NULL, 7
#define GRP2S_cl NULL, NULL, 8
#define GRP3b NULL, NULL, 9
#define GRP3S NULL, NULL, 10
#define GRP4  NULL, NULL, 11
#define GRP5  NULL, NULL, 12
#define GRP6  NULL, NULL, 13
#define GRP7 NULL, NULL, 14
#define GRP8 NULL, NULL, 15

#define FLOATCODE 50
#define FLOAT NULL, NULL, FLOATCODE

struct dis386 {
	char *name;
	int (*op1)();
	int bytemode1;
	int (*op2)();
	int bytemode2;
	int (*op3)();
	int bytemode3;
};

struct dis386 dis386[] = {
	{ "addb",	Eb, Gb }, /* 00 */
	{ "addS",	Ev, Gv },
	{ "addb",	Gb, Eb },
	{ "addS",	Gv, Ev },
	{ "addb",	AL, Ib },
	{ "addS",	eAX, Iv },
	{ "pushl",	es },
	{ "popl",	es },
	{ "orb",	Eb, Gb }, /* 08 */
	{ "orS",	Ev, Gv },
	{ "orb",	Gb, Eb },
	{ "orS",	Gv, Ev },
	{ "orb",	AL, Ib },
	{ "orS",	eAX, Iv },
	{ "pushl",	cs },
	{ "(bad)" },		/* 2-byte escape */

	{ "adcb",	Eb, Gb }, /* 10 */
	{ "adcS",	Ev, Gv },
	{ "adcb",	Gb, Eb },
	{ "adcS",	Gv, Ev },
	{ "adcb",	AL, Ib },
	{ "adcS",	eAX, Iv },
	{ "pushl",	ss },
	{ "popl",	ss },
	{ "sbbb",	Eb, Gb }, /* 18 */
	{ "sbbS",	Ev, Gv },
	{ "sbbb",	Gb, Eb },
	{ "sbbS",	Gv, Ev },
	{ "sbbb",	AL, Ib },
	{ "sbbS",	eAX, Iv },
	{ "pushl",	ds },
	{ "popl",	ds },

	{ "andb",	Eb, Gb }, /* 20 */
	{ "andS",	Ev, Gv },
	{ "andb",	Gb, Eb },
	{ "andS",	Gv, Ev },
	{ "andb",	AL, Ib },
	{ "andS",	eAX, Iv },
	{ "(bad)" },		/* (SEG=ES) */
	{ "daa" },
	{ "subb",	Eb, Gb }, /* 28 */
	{ "subS",	Ev, Gv },
	{ "subb",	Gb, Eb },
	{ "subS",	Gv, Ev },
	{ "subb",	AL, Ib },
	{ "subS",	eAX, Iv },
	{ "(bad)" },		/* (SEG=CS) */
	{ "das" },

	{ "xorb",	Eb, Gb }, /* 30 */
	{ "xorS",	Ev, Gv },
	{ "xorb",	Gb, Eb },
	{ "xorS",	Gv, Ev },
	{ "xorb",	AL, Ib },
	{ "xorS",	eAX, Iv },
	{ "(bad)" },
	{ "aaa" },
	{ "cmpb",	Eb, Gb },
	{ "cmpS",	Ev, Gv },
	{ "cmpb",	Gb, Eb },
	{ "cmpS",	Gv, Ev },
	{ "cmpb",	AL, Ib },
	{ "cmpS",	eAX, Iv },
	{ "(bad)" },
	{ "aas" },
	{ "incS",	eAX },
	{ "incS",	eCX },
	{ "incS",	eDX },
	{ "incS",	eBX },
	{ "incS",	eSP },
	{ "incS",	eBP },
	{ "incS",	eSI },
	{ "incS",	eDI },
	{ "decS",	eAX },
	{ "decS",	eCX },
	{ "decS",	eDX },
	{ "decS",	eBX },
	{ "decS",	eSP },
	{ "decS",	eBP },
	{ "decS",	eSI },
	{ "decS",	eDI },
	{ "pushS",	eAX },
	{ "pushS",	eCX },
	{ "pushS",	eDX },
	{ "pushS",	eBX },
	{ "pushS",	eSP },
	{ "pushS",	eBP },
	{ "pushS",	eSI },
	{ "pushS",	eDI },
	{ "popS",	eAX },
	{ "popS",	eCX },
	{ "popS",	eDX },
	{ "popS",	eBX },
	{ "popS",	eSP },
	{ "popS",	eBP },
	{ "popS",	eSI },
	{ "popS",	eDI },
	{ "pusha" },
	{ "popa" },
	{ "boundS",	Gv, Ma },
	{ "arpl",	Ew, Gw },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "pushS",	Iv },
	{ "imulS",	Gv, Ev, Iv },
	{ "pushl",	sIb },
	{ "imulS",	Gv, Ev, Ib },
	{ "insb",	Yb, indirDX },
	{ "insS",	Yv, indirDX },
	{ "outsb",	indirDX, Xb },
	{ "outsS",	indirDX, Xv },
	{ "jo",		Jb },
	{ "jno",	Jb },
	{ "jb",		Jb },
	{ "jae",	Jb },
	{ "je",		Jb },
	{ "jne",	Jb },
	{ "jbe",	Jb },
	{ "ja",		Jb },
	{ "js",		Jb },
	{ "jns",	Jb },
	{ "jp",		Jb },
	{ "jnp",	Jb },
	{ "jl",		Jb },
	{ "jnl",	Jb },
	{ "jle",	Jb },
	{ "jg",		Jb },
	{ GRP1b },
	{ GRP1S },
	{ "(bad)" },
	{ GRP1Ss },
	{ "testb",	Eb, Gb },
	{ "testS",	Ev, Gv },
	{ "xchgb",	Eb, Gb },
	{ "xchgS",	Ev, Gv },
	{ "movb",	Eb, Gb },
	{ "movS",	Ev, Gv },
	{ "movb",	Gb, Eb },
	{ "movS",	Gv, Ev },
	{ "movw",	Ew, Sw },
	{ "leaS",	Gv, M },
	{ "movw",	Sw, Ew },
	{ "popS",	Ev },
	{ "nop" },
	{ "xchgS",	eCX, eAX },
	{ "xchgS",	eDX, eAX },
	{ "xchgS",	eBX, eAX },
	{ "xchgS",	eSP, eAX },
	{ "xchgS",	eBP, eAX },
	{ "xchgS",	eSI, eAX },
	{ "xchgS",	eDI, eAX },
	{ "cwtl" },
	{ "cltd" },
	{ "lcall",	Ap },
	{ "(bad)" },
	{ "pushf" },
	{ "popf" },
	{ "sahf" },
	{ "lahf" },
	{ "movb",	AL, Ob },
	{ "movS",	eAX, Ov },
	{ "movb",	Ob, AL },
	{ "movS",	Ov, eAX },
	{ "movsb",	Yb, Xb },
	{ "movsS",	Yv, Xv },
	{ "cmpsb",	Yb, Xb },
	{ "cmpsS",	Yv, Xv },
	{ "testb",	AL, Ib },
	{ "testS",	eAX, Iv },
	{ "stosb",	Yb, AL },
	{ "stosS",	Yv, eAX },
	{ "lodsb",	AL, Xb },
	{ "lodsS",	eAX, Xv },
	{ "scasb",	AL, Xb },
	{ "scasS",	eAX, Xv },
	{ "movb",	AL, Ib },
	{ "movb",	CL, Ib },
	{ "movb",	DL, Ib },
	{ "movb",	BL, Ib },
	{ "movb",	AH, Ib },
	{ "movb",	CH, Ib },
	{ "movb",	DH, Ib },
	{ "movb",	BH, Ib },
	{ "movS",	eAX, Iv },
	{ "movS",	eCX, Iv },
	{ "movS",	eDX, Iv },
	{ "movS",	eBX, Iv },
	{ "movS",	eSP, Iv },
	{ "movS",	eBP, Iv },
	{ "movS",	eSI, Iv },
	{ "movS",	eDI, Iv },
	{ GRP2b },
	{ GRP2S },
	{ "ret",	Iw },
	{ "ret" },
	{ "lesS",	Gv, Mp },
	{ "ldsS",	Gv, Mp },
	{ "movb",	Eb, Ib },
	{ "movS",	Ev, Iv },
	{ "enter",	Iw, Ib },
	{ "leave" },
	{ "lret",	Iw },
	{ "lret" },
	{ "int3" },
	{ "int",	Ib },
	{ "into" },
	{ "iret" },
	{ GRP2b_one },
	{ GRP2S_one },
	{ GRP2b_cl },
	{ GRP2S_cl },
	{ "aam",	Ib },
	{ "aad",	Ib },
	{ "(bad)" },
	{ "xlat" },
	{ FLOAT },
	{ FLOAT },
	{ FLOAT },
	{ FLOAT },
	{ FLOAT },
	{ FLOAT },
	{ FLOAT },
	{ FLOAT },
	{ "loopne",	Jb },
	{ "loope",	Jb },
	{ "loop",	Jb },
	{ "jCcxz",	Jb },
	{ "inb",	AL, Ib },
	{ "inS",	eAX, Ib },
	{ "outb",	Ib, AL },
	{ "outS",	Ib, eAX },
	{ "call",	Av },
	{ "jmp",	Jv },
	{ "ljmp",	Ap },
	{ "jmp",	Jb },
	{ "inb",	AL, indirDX },
	{ "inS",	eAX, indirDX },
	{ "outb",	indirDX, AL },
	{ "outS",	indirDX, eAX },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "hlt" },
	{ "cmc" },
	{ GRP3b },
	{ GRP3S },
	{ "clc" },
	{ "stc" },
	{ "cli" },
	{ "sti" },
	{ "cld" },
	{ "std" },
	{ GRP4 },
	{ GRP5 },
};

struct dis386 dis386_twobyte[] = {
	{ GRP6 },
	{ GRP7 },
	{ "larS", Gv, Ew },
	{ "lslS", Gv, Ew },
	{ "(bad)" },
	{ "(bad)" },
	{ "clts" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "movl", Rd, Cd },
	{ "movl", Rd, Dd },
	{ "movl", Cd, Rd },
	{ "movl", Dd, Rd },
	{ "movl", Rd, Td },
	{ "(bad)" },
	{ "movl", Td, Rd },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "jo", Jv },
	{ "jno", Jv },
	{ "jb", Jv },
	{ "jae", Jv },
	{ "je", Jv },
	{ "jne", Jv },
	{ "jbe", Jv },
	{ "ja", Jv },
	{ "js", Jv },
	{ "jns", Jv },
	{ "jp", Jv },
	{ "jnp", Jv },
	{ "jl", Jv },
	{ "jge", Jv },
	{ "jle", Jv },
	{ "jg", Jv },
	{ "seto", Eb },
	{ "setno", Eb },
	{ "setb", Eb },
	{ "setae", Eb },
	{ "sete", Eb },
	{ "setne", Eb },
	{ "setbe", Eb },
	{ "seta", Eb },
	{ "sets", Eb },
	{ "setns", Eb },
	{ "setp", Eb },
	{ "setnp", Eb },
	{ "setl", Eb },
	{ "setge", Eb },
	{ "setle", Eb },
	{ "setg", Eb },
	{ "pushl", fs },
	{ "popl", fs },
	{ "(bad)" },
	{ "btS", Ev, Gv },
	{ "shldS", Ev, Gv, Ib },
	{ "shldS", Ev, Gv, CL },
	{ "(bad)" },
	{ "(bad)" },
	{ "pushl", gs },
	{ "popl", gs },
	{ "(bad)" },
	{ "btsS", Ev, Gv },
	{ "shrdS", Ev, Gv, Ib },
	{ "shrdS", Ev, Gv, CL },
	{ "(bad)" },
	{ "imulS", Gv, Ev },
	{ "(bad)" },
	{ "(bad)" },
	{ "lssS", Gv, Mp },
	{ "btrS", Ev, Gv },
	{ "lfsS", Gv, Mp },
	{ "lgsS", Gv, Mp },
	{ "movzbS", Gv, Eb },
	{ "movzwS", Gv, Ew },
	{ "(bad)" },
	{ "(bad)" },
	{ GRP8 },
	{ "btcS", Ev, Gv },
	{ "bsfS", Gv, Ev },
	{ "bsrS", Gv, Ev },
	{ "movsbS", Gv, Eb },
	{ "movswS", Gv, Ew },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },
	{ "(bad)" },  
};


static char *names32[]={
	"%eax","%ecx","%edx","%ebx", "%esp","%ebp","%esi","%edi",
};
static char *names16[] = {
	"%ax","%cx","%dx","%bx","%sp","%bp","%si","%di",
};
static char *names8[] = {
	"%al","%cl","%dl","%bl","%ah","%ch","%dh","%bh",
};
static char *names_seg[] = {
	"%es","%cs","%ss","%ds","%fs","%gs","%?","%?",
};
static char *names16_OP_E[] = {
	"%bx,%si", "%bx,%di", "%bp,%si", "%bp,%di", "%si", "%di","%bp","%bx",
};

struct dis386 grps[][8] = {
	{
		{ "addb",	Eb, Ib },
		{ "orb",	Eb, Ib },
		{ "adcb",	Eb, Ib },
		{ "sbbb",	Eb, Ib },
		{ "andb",	Eb, Ib },
		{ "subb",	Eb, Ib },
		{ "xorb",	Eb, Ib },
		{ "cmpb",	Eb, Ib }
	},
	{
		{ "addS",	Ev, Iv },
		{ "orS",	Ev, Iv },
		{ "adcS",	Ev, Iv },
		{ "sbbS",	Ev, Iv },
		{ "andS",	Ev, Iv },
		{ "subS",	Ev, Iv },
		{ "xorS",	Ev, Iv },
		{ "cmpS",	Ev, Iv }
	},
	{
		{ "addS",	Ev, sIb },
		{ "orS",	Ev, sIb },
		{ "adcS",	Ev, sIb },
		{ "sbbS",	Ev, sIb },
		{ "andS",	Ev, sIb },
		{ "subS",	Ev, sIb },
		{ "xorS",	Ev, sIb },
		{ "cmpS",	Ev, sIb }
	},
	{
		{ "rolb",	Eb, Ib },
		{ "rorb",	Eb, Ib },
		{ "rclb",	Eb, Ib },
		{ "rcrb",	Eb, Ib },
		{ "shlb",	Eb, Ib },
		{ "shrb",	Eb, Ib },
		{ "(bad)"		},
		{ "sarb",	Eb, Ib },
	},
	{
		{ "rolS",	Ev, Ib },
		{ "rorS",	Ev, Ib },
		{ "rclS",	Ev, Ib },
		{ "rcrS",	Ev, Ib },
		{ "shlS",	Ev, Ib },
		{ "shrS",	Ev, Ib },
		{ "(bad)" },
		{ "sarS",	Ev, Ib },
	},
	{
		{ "rolb",	Eb },
		{ "rorb",	Eb },
		{ "rclb",	Eb },
		{ "rcrb",	Eb },
		{ "shlb",	Eb },
		{ "shrb",	Eb },
		{ "(bad)" },
		{ "sarb",	Eb },
	},
	{
		{ "rolS",	Ev },
		{ "rorS",	Ev },
		{ "rclS",	Ev },
		{ "rcrS",	Ev },
		{ "shlS",	Ev },
		{ "shrS",	Ev },
		{ "(bad)" },
		{ "sarS",	Ev },
	},
	{
		{ "rolb",	Eb, CL },
		{ "rorb",	Eb, CL },
		{ "rclb",	Eb, CL },
		{ "rcrb",	Eb, CL },
		{ "shlb",	Eb, CL },
		{ "shrb",	Eb, CL },
		{ "(bad)" },
		{ "sarb",	Eb, CL },
	},
	{
		{ "rolS",	Ev, CL },
		{ "rorS",	Ev, CL },
		{ "rclS",	Ev, CL },
		{ "rcrS",	Ev, CL },
		{ "shlS",	Ev, CL },
		{ "shrS",	Ev, CL },
		{ "(bad)" },
		{ "sarS",	Ev, CL }
	},
	{
		{ "testb",	Eb, Ib },
		{ "(bad)",	Eb },
		{ "notb",	Eb },
		{ "negb",	Eb },
		{ "mulb",	AL, Eb },
		{ "imulb",	AL, Eb },
		{ "divb",	AL, Eb },
		{ "idivb",	AL, Eb }
	},
	{
		{ "testS",	Ev, Iv },
		{ "(bad)" },
		{ "notS",	Ev },
		{ "negS",	Ev },
		{ "mulS",	eAX, Ev },
		{ "imulS",	eAX, Ev },
		{ "divS",	eAX, Ev },
		{ "idivS",	eAX, Ev },
	},
	{
		{ "incb", Eb },
		{ "decb", Eb },
		{ "(bad)" },
		{ "(bad)" },
		{ "(bad)" },
		{ "(bad)" },
		{ "(bad)" },
		{ "(bad)" },
	},
	{
		{ "incS",	Ev },
		{ "decS",	Ev },
		{ "call",	indirEv },
		{ "lcall",	indirEv },
		{ "jmp",	indirEv },
		{ "ljmp",	indirEv },
		{ "pushS",	Ev },
		{ "(bad)" },
	},
	{
		{ "sldt",	Ew },
		{ "str",	Ew },
		{ "lldt",	Ew },
		{ "ltr",	Ew },
		{ "verr",	Ew },
		{ "verw",	Ew },
		{ "(bad)" },
		{ "(bad)" 	}
	},
	{
		{ "sgdt", Ew },
		{ "sidt", Ew },
		{ "lgdt", Ew },
		{ "lidt", Ew },
		{ "smsw", Ew },
		{ "(bad)" },
		{ "lmsw", Ew },
		{ "(bad)" },
	},
	{
		{ "(bad)" },
		{ "(bad)" },
		{ "(bad)" },
		{ "(bad)" },
		{ "btS",	Ev, Ib },
		{ "btsS",	Ev, Ib },
		{ "btrS",	Ev, Ib },
		{ "btcS",	Ev, Ib },
	}
};

#define PREFIX_REPZ 1
#define PREFIX_REPNZ 2
#define PREFIX_LOCK 4
#define PREFIX_CS 8
#define PREFIX_SS 0x10
#define PREFIX_DS 0x20
#define PREFIX_ES 0x40
#define PREFIX_FS 0x80
#define PREFIX_GS 0x100
#define PREFIX_DATA 0x200
#define PREFIX_ADR 0x400
#define PREFIX_FWAIT 0x800

